export const versions = [
  {
    id: 249,
    dashboardId: 74,
    parentVersion: 10,
    restoredFrom: 0,
    version: 11,
    created: '2021-01-15T14:44:44+01:00',
    createdBy: 'admin',
    message: 'Another day another change...',
  },
  {
    id: 247,
    dashboardId: 74,
    parentVersion: 9,
    restoredFrom: 0,
    version: 10,
    created: '2021-01-15T10:19:17+01:00',
    createdBy: 'admin',
    message: '',
  },
  {
    id: 246,
    dashboardId: 74,
    parentVersion: 8,
    restoredFrom: 0,
    version: 9,
    created: '2021-01-15T10:18:12+01:00',
    createdBy: 'admin',
    message: '',
  },
  {
    id: 245,
    dashboardId: 74,
    parentVersion: 7,
    restoredFrom: 0,
    version: 8,
    created: '2021-01-15T10:11:16+01:00',
    createdBy: 'admin',
    message: '',
  },
  {
    id: 239,
    dashboardId: 74,
    parentVersion: 6,
    restoredFrom: 0,
    version: 7,
    created: '2021-01-14T15:14:25+01:00',
    createdBy: 'admin',
    message: '',
  },
  {
    id: 237,
    dashboardId: 74,
    parentVersion: 5,
    restoredFrom: 0,
    version: 6,
    created: '2021-01-14T14:55:29+01:00',
    createdBy: 'admin',
    message: '',
  },
  {
    id: 236,
    dashboardId: 74,
    parentVersion: 4,
    restoredFrom: 0,
    version: 5,
    created: '2021-01-14T14:28:01+01:00',
    createdBy: 'admin',
    message: '',
  },
  {
    id: 218,
    dashboardId: 74,
    parentVersion: 3,
    restoredFrom: 0,
    version: 4,
    created: '2021-01-08T10:45:33+01:00',
    createdBy: 'admin',
    message: '',
  },
  {
    id: 217,
    dashboardId: 74,
    parentVersion: 2,
    restoredFrom: 0,
    version: 3,
    created: '2021-01-05T15:41:33+01:00',
    createdBy: 'admin',
    message: '',
  },
  {
    id: 216,
    dashboardId: 74,
    parentVersion: 1,
    restoredFrom: 0,
    version: 2,
    created: '2021-01-05T15:01:50+01:00',
    createdBy: 'admin',
    message: '',
  },
  {
    id: 215,
    dashboardId: 74,
    parentVersion: 1,
    restoredFrom: 0,
    version: 1,
    created: '2021-01-05T14:59:15+01:00',
    createdBy: 'admin',
    message: '',
  },
];
