.navbar-page-btn {
  text-overflow: ellipsis;
  overflow: hidden;
  white-space: nowrap;
  display: flex;
  align-items: center;
  margin: 0;
  font-size: $font-size-lg;
  min-height: $navbarHeight;

  .gicon {
    top: -2px;
    position: relative;
    font-size: 17px;
    line-height: 8px;
    opacity: 0.75;
    margin-right: 10px;
    display: none;

    @include media-breakpoint-up(md) {
      display: inline-block;
    }
  }

  .navbar-page-btn__folder-icon {
    display: none;

    @include media-breakpoint-up(md) {
      display: inline-block;
    }
  }
}
