[[Subject .Subject "Welcome to Grafana"]]

<table class="row">
	<tr>
		<td class="wrapper last">

			<table class="twelve columns">
				<tr>
					<td>
						<h4>Hi [[.Name]],</h4>
					</td>
					<td class="expander"></td>
				</tr>
				<tr>
					<td>
						Welcome! Ready to start building some beautiful metric and analytic dashboards?
					</td>
				</tr>
			</table>

		</td>
	</tr>
</table>

<table class="row">
	<tr>
		<td class="wrapper last">
			<table class="twelve columns">
				<tr>
					<td class="center">
						<p>
							If you are new to Grafana please read the <a href="https://grafana.com/docs/grafana/latest/getting-started/getting-started/">Getting Started</a> guide.
						</p>
					</td>
					<td class="expander"></td>
				</tr>
				<tr>
					<td>
						Thank you for joining our community.
						<br />
						<p>The Grafana Team</p>
					</td>
				</tr>
			</table>
		</td>
	</tr>
</table>

