.dashlist-section-header {
  margin-bottom: $spacer;
  color: $text-color-weak;
}

.dashlist-section {
  margin-bottom: $spacer;
  padding-top: 3px;
}

.dashlist-link {
  @include list-item();
  display: flex;

  .fa {
    padding-top: 3px;
  }

  .fa-star {
    color: $orange;
  }
}

.dashlist-star {
  display: flex;
  align-items: center;
  color: $text-color-weak;
}

.dashlist-folder {
  color: $text-color-weak;
  font-size: $font-size-xs;
}

.dashlist-link-body {
  flex-grow: 1;
}
