<?xml version="1.0" encoding="utf-8"?>
<Wix xmlns="http://schemas.microsoft.com/wix/2006/wi"
     xmlns:fire="http://schemas.microsoft.com/wix/FirewallExtension">
  <Fragment>
      <ComponentGroup Id="OracleEnvironment">
        <Component Id="ORACLE_ENV" Guid="13ed28fd-a2d8-45e3-9c0f-8ec8d9e3ad16" Directory="INSTALLDIR">
          <Environment Id="OCI_LIB64" Name="OCI_LIB64" Value="[INSTALLDIR]instantclient_{{ instant_client_version }}" Permanent="yes" Part="last" Action="set" System="yes" />
          <Environment Id="PATH" Name="PATH" Value="[INSTALLDIR]instantclient_{{instant_client_version}}" Permanent="yes" Part="last" Action="set" System="yes" />
        </Component>
      </ComponentGroup>
  </Fragment>
</Wix>
