export interface DashListOptions {
  showStarred: boolean;
  showRecentlyViewed: boolean;
  showSearch: boolean;
  showHeadings: boolean;
  maxItems: number;
  query: string;
  folderId: number;
  tags: string[];
}
