<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
	<meta name="viewport" content="width=device-width"/>
	<!-- build:css css/tidy.css -->
	<link inline rel="stylesheet" href="../assets/css/ink.css">
	<link inline rel="stylesheet" href="../assets/css/style.css">
	<!-- /build -->
</head>
<body leftmargin="0" topmargin="0" marginwidth="0" marginheight="0" class="main" style="background: #2e2e2e; height: 100% !important; margin: 0 auto; padding: 0; width: 100% !important" bgcolor="#2e2e2e"><style type="text/css">
body {
	height: 100% !important; width: 100% !important;
}
body .copy {
   -ms-text-size-adjust: 100%; -webkit-text-size-adjust: 100%;
}
.ExternalClass {
	width: 100%;
}
.ExternalClass {
	line-height: 100%;
}
img {
    -ms-interpolation-mode: bicubic;
}
img {
	border: 0 !important; outline: none !important; text-decoration: none !important;
}
a:hover {
	text-decoration: underline;
}
td[class="stack-column-center"] {
    text-align: center !important;
}
@media (max-width: 600px) {
  table[class="email-container"] {
    width: 95% !important;
  }
  img[class="fluid"] {
    width: 100% !important; max-width: 100% !important; height: auto !important; margin: auto !important;
  }
  img[class="fluid-centered"] {
    width: 100% !important; max-width: 100% !important; height: auto !important; margin: auto !important;
  }
  img[class="fluid-centered"] {
    margin: auto !important;
  }
  td[class="comms-content"] {
    padding: 20px !important;
  }
  td[class="stack-column"] {
    display: block !important; width: 100% !important; direction: ltr !important;
  }
  td[class="stack-column-center"] {
    display: block !important; width: 100% !important; direction: ltr !important;
  }
  td[class="stack-column-center"] {
    text-align: center !important;
  }
  td[class="copy"] {
    font-size: 14px !important; line-height: 24px !important; padding: 0 30px !important;
  }
  td[class="copy -center"] {
    font-size: 14px !important; line-height: 24px !important; padding: 0 30px !important;
  }
  td[class="copy -bold"] {
    font-size: 14px !important; line-height: 24px !important; padding: 0 30px !important;
  }
  td[class="small-text"] {
    font-size: 14px !important; line-height: 24px !important; padding: 0 30px !important;
  }
  td[class="mini-centered-text"] {
    font-size: 14px !important; line-height: 24px !important; padding: 15px 30px !important;
  }
  td[class="copy -padd"] {
    padding: 0 40px !important;
  }
  span[class="sep"] {
    display: none !important;
  }
  td[class="mb-hide"] {
    display: none !important; height: 0 !important;
  }
  td[class="spacer mb-shorten"] {
    height: 25px !important;
  }
  .two-up td {
  	width:270px;
  }
}
</style>

	<table class="body" style="background: #2e2e2e;">
		<tr>
			<td class="center" align="center" valign="top">
        <center>
					<table class="row header">
						<tr>
						  <td class="center" align="center">
						    <center>

						      <table class="container">
						        <tr>
						          <td class="wrapper last">

						            <table class="twelve columns">
						              <tr>
						                <td class="twelve sub-columns center">
                              <img class="logo" src="https://grafana.com/assets/img/logo_new_transparent_200x48.png" style="width: 200px; float: none; display: inline">
                            </td>
                            <td class="expander"></td>
                          </tr>
						            </table>

						          </td>
						        </tr>
						      </table>

						    </center>
						  </td>
						</tr>
					</table>

					<table class="container" style="background-color: #efefef;" width="600">
						<tr>
							<td height="2" class="spacer mb-shorten" style="font-size: 0; line-height: 0; mso-table-lspace: 0pt; mso-table-rspace: 0pt; vertical-align: top; border: 0; background: linear-gradient(to right, #ffed00 0%, #f26529 75%); height: 2px !important" valign="top"> </td>
						</tr>
						<tr>
							<td class="mini-centered-text" style="color: #343b41; font: 400 16px/27px 'Helvetica Neue', Helvetica, Arial, sans-serif; mso-table-lspace: 0pt; mso-table-rspace: 0pt; padding: 25px 35px; text-align: center; vertical-align: top" align="center" valign="top">
								{{> body }}
								<!-- container end below -->
							</td>
						</tr>
					</table>
					<!-- footer -->
					<table class="footer center">
						<tr>
							<td class="wrapper last">
								<table class="twelve columns center">
									<tr>
										<td class="twelve" align="center">
											<center>
												<p style="text-align: center; font-size: 12px; color: #999999;">
													Sent by <a href="[[.AppUrl]]">Grafana v[[.BuildVersion]]</a>
													<br />&copy; 2019 Grafana Labs
												</p>
											</center>
										</td>
										<td class="expander"></td>
									</tr>
								</table>
							</td>
						</tr>
					</table>
				</center>
			</td>
		</tr>
	</table>
</body>
</html>
