<loki-annotations-query-editor
  expr="ctrl.annotation.expr"
  max-lines="ctrl.annotation.maxLines"
  instant="ctrl.annotation.instant"
  on-change="ctrl.onQueryChange"
  datasource="ctrl.datasource"
/>
