<!-- This email is sent when an existing user is added to an organization -->

[[Subject .Subject "[[.InvitedBy]] has added you to the [[.OrgName]] organization"]]

<table class="row">
	<tr>
		<td class="wrapper last">

			<table class="twelve columns">
				<tr>
					<td>
						<h4 class="center">You have been added to [[.OrgName]]</h4>
					</td>
					<td class="expander"></td>
				</tr>
			</table>

		</td>
	</tr>
</table>

<table class="row">
	<tr>
		<td class="wrapper last">
			<table class="twelve columns">
				<tr>
					<td class="center">
						<p><b>[[.InvitedBy]]</b> has added you to the <b>[[.OrgName]]</b> organization in Grafana.
						<p>Once logged in, [[.OrgName]] will be available in the left side menu, in the dropdown below your username.</p>
					</td>
					<td class="expander"></td>
				</tr>
				<tr>
					<td class="center">
						<table class="better-button" align="center" border="0" cellspacing="0" cellpadding="0">
							<tr>
								<td align="center" class="better-button" bgcolor="#ff8f2b"><a href="[[.AppUrl]]" target="_blank">Log in now</a></td>
							</tr>
						</table>
					</td>
				</tr>
			</table>
		</td>
	</tr>
</table>


