/**
 * A coordinate on a two dimensional plane.
 */
export interface CartesianCoords2D {
  x: number;
  y: number;
}
/**
 * 2d object dimensions.
 */
export interface Dimensions2D {
  width: number;
  height: number;
}
