<!-- This email is sent when user who does not already exist in Grafana is added to an organization -->

[[Subject .Subject "[[.InvitedBy]] has invited you to join Grafana"]]

<table class="row">
	<tr>
		<td class="wrapper last">

			<table class="twelve columns">
				<tr>
					<td>
						<h4 class="center">You're invited to join [[.OrgName]]</h4>
					</td>
					<td class="expander"></td>
				</tr>
			</table>

		</td>
	</tr>
</table>

<table class="row">
	<tr>
		<td class="wrapper last">
			<table class="twelve columns">
				<tr>
					<td class="center">
						<p>You've been invited to join the <b>[[.OrgName]]</b> organization by <b>[[.InvitedBy]]</b>. To accept your invitation and join the team, please click the link below:</p>
					</td>
				</tr>
				<tr>				
					<td class="center">
	                    <table class="better-button" align="center" border="0" cellspacing="0" cellpadding="0">
	                    	<tr>
	                          <td align="center" class="better-button" bgcolor="#ff8f2b"><a href="[[.LinkUrl]]" target="_blank">Accept Invitation</a></td>
	                        </tr>
	                    </table>
					</td>
				</tr>
				<tr>
					<td class="center">
						<p>You can also copy/paste this link into your browser directly: <a href="[[.LinkUrl]]">[[.LinkUrl]]</a></p>
					</td>								
					<td class="expander"></td>
				</tr>
			</table>
		</td>
	</tr>
</table>