export const renderGeneratedFileBanner = (themeFile: string, templateFile: string) => `/***
 * !!! THIS FILE WAS GENERATED AUTOMATICALLY !!!
 *
 * Do not modify this file!
 * - Edit ${themeFile} to regenerate
 * - Edit ${templateFile} to update template
 *
 * !!! THIS FILE WAS GENERATED AUTOMATICALLY !!!
 */
`;
