.panel-wrapper {
  height: 100%;
  position: relative;

  &--view {
    flex: 1 1 0;
    height: 90%;
  }
}

.panel-editor__scroll {
  flex-grow: 1;
  min-width: 0;
  display: flex;
  min-height: 0;
  height: 100%;
  overflow: hidden;
}

.panel-editor__content {
  padding: 0 16px 16px 16px;
}

.panel-in-fullscreen {
  .search-container {
    left: 0 !important;
  }
}

.ds-picker {
  position: relative;
  min-width: 200px;
  max-width: 300px;
}

/* Used by old angular panels */
.panel-options-group {
  border-bottom: $panel-border;
}

.panel-options-group__header {
  padding: 8px 16px 8px 8px;
  position: relative;
  display: flex;
  align-items: center;
  cursor: pointer;
  font-weight: 500;
  color: $text-color-semi-weak;

  &:hover {
    color: $text-color;

    .panel-options-group__icon {
      color: $text-color;
    }
  }
}

.panel-options-group__icon {
  color: $text-color-weak;
  margin-right: 8px;
}

.panel-options-group__title {
  position: relative;
}

.panel-options-group__body {
  padding: 8px 16px 16px 32px;
}
