[[Subject .Subject "Reset your Grafana password - [[.Name]]"]]

<table class="row">
	<tr>
		<td class="wrapper last">

			<table class="twelve columns">
				<tr>
					<td>
						<h4>Hi [[.Name]],</h4>
					</td>
					<td class="expander"></td>
				</tr>
			</table>

		</td>
	</tr>
</table>

<table class="row">
	<tr>
		<td class="wrapper last">
			<table class="twelve columns">
				<tr>
					<td class="center">
						<p>
							Please click the following link to reset your password within <b>[[.EmailCodeValidHours]] hours</b>.
						</p>
						<p>
							<a href="[[.AppUrl]]user/password/reset?code=[[.Code]]">[[.AppUrl]]user/password/reset?code=[[.Code]]</a>
						</p>
						<p>Not working? Try copying and pasting it to your browser.</p>
					</td>
					<td class="expander"></td>
				</tr>
			</table>

		</td>
	</tr>
</table>


