[[Subject .Subject "Welcome to Grafana, please complete your sign up!"]]

<table class="row">
	<tr>
		<td class="wrapper last">

			<table class="twelve columns">
				<tr>
					<td>
						<h4 class="center">Complete the signup</h4>
					</td>
					<td class="expander"></td>
				</tr>
			</table>

		</td>
	</tr>
</table>

<table class="row">
	<tr>
		<td class="wrapper last">
			<table class="twelve columns">
				<tr>
					<td class="center">
						Copy and paste the email verification code:<br>
						<span class="verification-code">[[.Code]]</span><br> in
						the sign up form <strong>or</strong> use the link below.
					</td>
					<td class="expander"></td>
				</tr>
				<tr>
					<td class="center">
						<table class="better-button" align="center" border="0" cellspacing="0" cellpadding="0">
							<tr>
								<td align="center" class="better-button" bgcolor="#ff8f2b"><a href="[[.SignUpUrl]]" target="_blank">Complete Sign Up</a></td>
							</tr>
						</table>
					</td>
				</tr>
			</table>
		</td>
	</tr>
</table>


